@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: hostthre.c,v 1.47 2007-02-26 04:24:26 giva Exp $
+ * $Id: hostthre.c,v 1.49 2007-06-11 04:33:21 giva Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -179,28 +179,21 @@ struct thread_sync_data {
 static
 void destroy_thread_sync_data(struct thread_sync_data * tsd)
 {
-  if (tsd->hostname) {
+  if (tsd->hostname)
     free(tsd->hostname);
-    tsd->hostname = NULL;
-  }
-  if (tsd->event_terminate) {
+  if (tsd->event_terminate)
     CloseHandle(tsd->event_terminate);
-    tsd->event_terminate = NULL;
-  }
-  if (tsd->mutex_terminate) {
+  if (tsd->mutex_terminate)
     CloseHandle(tsd->mutex_terminate);
-    tsd->mutex_terminate = NULL;
-  }
-  if (tsd->mutex_waiting) {
+  if (tsd->mutex_waiting)
     CloseHandle(tsd->mutex_waiting);
-    tsd->mutex_waiting = NULL;
-  }
+  memset(tsd,0,sizeof(*tsd));
 }
 
 /* Initialize resolver thread synchronization data */
 static
 BOOL init_thread_sync_data(struct thread_data * td,
-                           char * hostname,
+                           const char * hostname,
                            struct thread_sync_data * tsd)
 {
   HANDLE curr_proc = GetCurrentProcess();
@@ -293,6 +286,7 @@ static unsigned __stdcall gethostbyname_thread (void *arg)
    * due to a resolver timeout.
    */
   struct thread_sync_data tsd = { 0,0,0,NULL };
+
   if (!init_thread_sync_data(td, conn->async.hostname, &tsd)) {
     /* thread synchronization data initialization failed */
     return (unsigned)-1;
@@ -353,6 +347,7 @@ static unsigned __stdcall getaddrinfo_thread (void *arg)
    * due to a resolver timeout.
    */
   struct thread_sync_data tsd = { 0,0,0,NULL };
+
   if (!init_thread_sync_data(td, conn->async.hostname, &tsd)) {
     /* thread synchronization data initialization failed */
     return -1;
@@ -595,7 +590,7 @@ CURLcode Curl_wait_for_resolv(struct connectdata *conn,
   timeout =
     conn->data->set.connecttimeout ? conn->data->set.connecttimeout :
     conn->data->set.timeout ? conn->data->set.timeout :
-    CURL_TIMEOUT_RESOLVE; /* default name resolve timeout */
+    CURL_TIMEOUT_RESOLVE * 1000; /* default name resolve timeout */
   ticks = GetTickCount();
 
   /* wait for the thread to resolve the name */
