@@ -18,7 +18,7 @@
  * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
  * KIND, either express or implied.
  *
- * $Id: sendf.c,v 1.127 2007-03-21 13:09:39 yangtse Exp $
+ * $Id: sendf.c,v 1.129 2007-05-02 11:14:55 bagder Exp $
  ***************************************************************************/
 
 #include "setup.h"
@@ -79,11 +79,11 @@ static struct curl_slist *slist_get_last(struct curl_slist *list)
 }
 
 /*
- * curl_slist_append() appends a string to the linked list. It always retunrs
- * the address of the first record, so that you can sure this function as an
+ * curl_slist_append() appends a string to the linked list. It always returns
+ * the address of the first record, so that you can use this function as an
  * initialization function as well as an append function. If you find this
  * bothersome, then simply create a separate _init function and call it
- * appropriately from within the proram.
+ * appropriately from within the program.
  */
 struct curl_slist *curl_slist_append(struct curl_slist *list,
                                      const char *data)
@@ -495,7 +495,7 @@ int Curl_read(struct connectdata *conn, /* connection data */
     }
     /* If we come here, it means that there is no data to read from the buffer,
      * so we read from the socket */
-    bytesfromsocket = MIN(sizerequested, sizeof(conn->master_buffer));
+    bytesfromsocket = MIN(sizerequested, BUFSIZE * sizeof (char));
     buffertofill = conn->master_buffer;
   }
   else {
